#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tankobon', 'tankobon.bootstraps']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.1',
 'click>=7.1.2',
 'coloredlogs>=14.0',
 'html5lib>=1.1',
 'requests>=2.24.0',
 'requests-random-user-agent>=2020.10.19']

extras_require = \
{'gui': ['pyglet>=1.5.8'], 'jsonschema': ['jsonschema>=3.2.0']}

entry_points = \
{'console_scripts': ['tankobon = tankobon.cli:cli']}

setup(name='tankobon',
      version='2.0.0a3',
      description='Manga browser/downloader.',
      author='Ong Yong Xin',
      author_email='ongyongxin2020+github@gmail.com',
      url='https://github.com/ongyx/tankobon',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
